@@ -1,4 +1,4 @@
-/// Thomas Nagy 2007-2009 GPLV3
+/// Thomas Nagy 2007-2012 GPLV3
 
 #ifndef CANVAS_FLAG_H
 #define CANVAS_FLAG_H
@@ -19,8 +19,7 @@ class canvas_flag: public QGraphicsRectItem
 	public:
 		canvas_flag(canvas_view *i_oGraphWidget, canvas_item *i_oFrom);
 
-		//enum { Type = gratype(797) };
-		int type() const { return gratype(797); }
+		int type() const { return CANVAS_FLAG_T; }
 		canvas_view *m_oGraph;
 		canvas_item *m_oItem;
 		void paint(QPainter *i_oPainter, const QStyleOptionGraphicsItem *i_oOption, QWidget * i_oW);
